# -*- coding: utf-8 -*-
import random
import time
import hmac
import hashlib
import base64

from .compat import quote


class Auth(object):
    def __init__(self, secret_id, secret_key, appid, bucket):
        self.secret_id = secret_id
        self.secret_key = secret_key
        self.appid = appid
        self.bucket = bucket

    def _sign(self, cos_path, expired):
        """expired为0时是multi签名方式"""
        rdm = random.randint(0, 999999999)

        if cos_path:
            fileid = '/{}/{}{}'.format(self.appid,
                                       self.bucket,
                                       quote(cos_path, '/'))
        else:
            fileid = ''

        now = int(time.time())
        if expired and expired <= 7776000:
            expired = now + expired

        sign_items = [
            ('a', self.appid),
            ('b', self.bucket),
            ('k', self.secret_id),
            ('t', now),
            ('e', expired),
            ('r', rdm),
            ('f', fileid)
        ]

        plain_text = '&'.join('{}={}'.format(*x) for x in sign_items)
        plain_bin = plain_text.encode('utf-8')

        sha1_hmac = hmac.new(self.secret_key.encode('utf-8'),
                             plain_bin,
                             hashlib.sha1)

        return base64.b64encode(sha1_hmac.digest() + plain_bin)

    def sign_once(self, cos_path):
        """单次签名(针对删除和更新操作)
        :param cos_path: 要操作的cos路径, 以'/'开始
        :return: 签名字符串
        """
        return self._sign(cos_path, 0)

    def sign_multi(self, expired):
        """多次签名(针对上传文件，创建目录, 获取文件目录属性, 拉取目录列表)
        :param expired: 签名过期时间, UNIX时间戳, 或者过期时长
        :return: 签名字符串
        """
        return self._sign('', expired)



# -*- coding: utf-8 -*-
import time
import requests
from .auth import Auth


class COSClient(object):

    def __init__(self, secret_id, secret_key, region, appid, bucket):
        self.secret_id = secret_id
        self.secret_key = secret_key
        self.appid = appid
        self.bucket = bucket
        self.auth = Auth(secret_id, secret_key, appid, bucket)

        self.url_prefix = 'http://{}.file.myqcloud.com/files/v2/{}/{}' \
            .format(region, appid, bucket)

        self.session = requests.Session()
        self._reset_auth()

    def _reset_auth(self):
        self.auth_expired = int(time.time()) + 21600
        self.session.headers['Authorization'] = \
            self.auth.sign_multi(self.auth_expired)

    def _request(self, method, cos_path,
                 params=None, data=None, headers=None, files=None):
        if int(time.time()) > self.auth_expired:
            self._reset_auth()
        if not cos_path.startswith('/'):
            cos_path = '/' + cos_path

        url = '{}{}'.format(self.url_prefix, cos_path)
        r = self.session.request(method, url,
                                 params=params, data=data,
                                 headers=headers, files=files)
        if r.json().get('code') == -96:
            self._reset_auth()
        return r

    def upload_local(self, local_path, cos_path,
                     sha=None, biz_attr=None, insertOnly=None):
        filecontent = open(local_path, 'rb')
        return self.upload_content(
            filecontent,
            cos_path,
            sha,
            biz_attr,
            insertOnly
        )

    def upload_content(self, filecontent, cos_path,
                       sha=None, biz_attr=None, insertOnly=None):
        """`简单上传文件
        <https://www.qcloud.com/document/api/436/6066>`_
        """
        data = {'op': 'upload'}
        files = {'filecontent': filecontent}

        if sha is not None:
            data['sha'] = sha

        if biz_attr is not None:
            data['biz_attr'] = biz_attr

        if insertOnly is not None:
            data['insertOnly'] = insertOnly

        return self._request('POST', cos_path, data=data, files=files)

    def stat(self, cos_path):
        """`查询文件属性
        <https://www.qcloud.com/document/api/436/6069>`_
        """
        return self._request('GET', cos_path, params={'op': 'stat'})



# -*- coding: utf-8 -*-
import argparse
import os.path
from termcolor import cprint

from .configure import QcloudConfig
from .client import COSClient

parser = argparse.ArgumentParser(
    description='腾讯云COS管理.')
parser.add_argument('local_dir')
parser.add_argument('appid')
parser.add_argument('bucket')
parser.add_argument('cos_dir')


def main():
    # 参数包含local_path, bucket
    parsed_args = parser.parse_args()

    # 命令行参数
    local_dir = parsed_args.local_dir
    appid = parsed_args.appid
    bucket = parsed_args.bucket
    cos_dir = parsed_args.cos_dir

    local_dir = os.path.abspath(os.path.expanduser(local_dir))
    slice_start = len(local_dir) + 1

    print('local_dir: ', local_dir)
    print('appid: ', appid, 'bucket: ', bucket, 'cos_dir: ', cos_dir)

    # 配置文件配置
    config = QcloudConfig()
    client = COSClient(config.secret_id, config.secret_key, config.region,
                       appid, bucket)

    for root, dirs, files in os.walk(local_dir):
        for f in files:
            local_path = os.path.join(root, f)
            cos_path = os.path.join(cos_dir, local_path[slice_start:])
            if local_path.endswith('.html'):
                r = client.upload_local(local_path, cos_path, insertOnly=0)
            else:
                r = client.upload_local(local_path, cos_path)
            j = r.json()
            code = j.get('code')
            if code == 0:
                cprint('{} Upload success!'.format(cos_path), 'green')
            elif code == -177:
                cprint('{} Upload failed, file exit.'.format(cos_path), 'blue')
            else:
                cprint('{} Upload failed! {}'.format(cos_path, r.text), 'red')



#!/usr/bin/env python
from setuptools import setup, find_packages


with open('README.rst') as f:
    readme = f.read()


setup(
    name='qcos',
    version='0.1.1',
    description='腾讯云对象存储库，支持命令行',
    long_description=readme,
    author='codeif',
    author_email='me@codeif.com',
    url='https://github.com/codeif/qcos',
    license='MIT',
    entry_points={
        'console_scripts': [
            'qcos = qcos.cli:main',
        ],
    },
    install_requires=['requests', 'termcolor'],
    packages=find_packages(exclude=("tests", "tests.*")),
)



# -*- coding: utf-8 -*-

"""
qcos.compat
~~~~~~~~~~~~~~~
This module handles import compatibility issues between Python 2 and
Python 3.
"""

import sys

# -------
# Pythons
# -------

# Syntax sugar.
_ver = sys.version_info

#: Python 2.x?
is_py2 = (_ver[0] == 2)

#: Python 3.x?
is_py3 = (_ver[0] == 3)

# ---------
# Specifics
# ---------

if is_py2:
    from urllib import quote
    from ConfigParser import ConfigParser

elif is_py3:
    from urllib.parse import quote
    from configparser import ConfigParser


__all__ = ['quote', 'ConfigParser']



# -*- coding: utf-8 -*-
import os
import os.path

from .compat import ConfigParser


class QcloudConfig(object):
    def __init__(self):
        config = ConfigParser()
        config.read([os.path.expanduser('~/.qcloudcli/configure'),
                     os.path.expanduser('~/.qcloudcli/credentials'), ])
        section = 'default'
        self.secret_id = config.get(section, 'qcloud_secretid')
        self.secret_key = config.get(section, 'qcloud_secretkey')
        self.region = config.get(section, 'region')
